==> kanso-orm-0.0.12.tgz: package/lib/model-factory.js <==
'use strict';
var QueryObject = require('./query-object');
var Relationship = require('./relationship');

function ModelFactory() {
  this.ORM = null;
  this.Model = null;
  this.tableName = '';
  this.attributes = {};
  this.relationships = {};
  this.pre = {};
  this.post = {};
  this.static = {};
  this.method = {};
}

//
// Model Operations
//

ModelFactory.prototype.extend = function(ORM, tableName, data) {
  this.ORM = ORM;
  this.tableName = tableName;
  this.Model = this.createModel(data.method);
  this.addPre(data.pre);
  this.addPost(data.post);
  this.addStatic(data.static);
  this.addTableAttributes(data.attributes);
  console.log('Creating ModelFactory: ', this.tableName);
  return this;
};

ModelFactory.prototype.createModel = function(methods) {

  //
  // Create base
  var Model = function() {};

  //
  // Add methods to model prototype
  for(var method in methods) {
    Model.prototype[method] = methods[method];
  }

  //
  // Return
  return Model;
};

ModelFactory.prototype.addPre = function(pre) {
  for(var method in pre) {
    this.pre[method] = pre[method];
  }
};

ModelFactory.prototype.addPost = function(post) {
  for(var method in post) {
    this.post[method] = post[method];
  }
};

ModelFactory.prototype.addStatic = function(statics) {
  for(var method in statics) {
    this[method] = statics[method];
  }
};

ModelFactory.prototype.addTableAttributes = function(attributes) {
  for(var attribute in attributes) {
    switch(typeof attributes[attribute]) {
      case 'object': {
        this.attributes[attribute] = attributes[attribute];
        break;
      }
      case 'function': {
        this.relationships[attribute] = attributes[attribute](this);
        break;
      }
    }
  }
};

//
// Model Instance Operations
//

ModelFactory.prototype.initializeModel = function(data, created) {
  var model = new this.Model();
  // Set all attributes
  for(var attribute in data) {
    model[attribute] = data[attribute];
  }
  if(created) {
    model.created = created;
  }
  // Set all model methods
  return model;
};

ModelFactory.prototype.initializeModels = function(data) {
  var self = this;
  var models = data.map(function(d) {
    return self.initializeModel(d);
  });
  return models;
};

//
// Attribute Operations
//

ModelFactory.prototype.getReturnAttributes = function() {
  var returnAttributes = [];
  for(var attribute in this.attributes) {
    if(!this.attributes[attribute].private) {
      returnAttributes.push(attribute);
    }
  }
  return returnAttributes;
};

//
// Relationship Operations
//

//
// Private
//

// TODO: Make private
ModelFactory.prototype.includeForMany = function(models, include) {
  var promises = [];
  models.forEach(function(model) {
    promises.push(this.includeForOne(model, include));
  }, this);
  return Promise.all(promises);
};

// TODO: Make private
ModelFactory.prototype.includeForOne = function(model, include) {
  var self = this;
  var promises = [];
  include.forEach(function(relationshipTableName) {
    var relationship = self.relationships[relationshipTableName];
    if(self.relationships.hasOwnProperty(relationshipTableName)) {
      var promise = relationship.include(relationshipTableName, model);
      promises.push(promise);
    }
  });
  return Promise.all(promises).then(function() {
    return model;
  });
};

//
// Query Operations
//

ModelFactory.prototype.hook = function(hook, data) {
  return new Promise(function(resolve, reject) {
    if(hook) {
      hook(data, resolve);
    } else {
      resolve(data);
    }
  });
};

ModelFactory.prototype.create = function(where, options) {
  var model, query, initial;

  // TEMP
  // Start transaction
  if(!options) {
    options = {};
    if(!options.transaction) {
      initial = true;
      options.transaction = this.ORM.querier.transaction();
      options.table = this.tableName;
      options.depth = 0;
    }
  }
  return options.transaction
  .then(() => {
    return this.hook(this.pre.create, where);
  })
  // Pre
  .then((newWhere) => {
    where = Object.assign({}, where, newWhere);
    query = new QueryObject(this.tableName, where, this.relationships, options);
  })
  // Create belongs to one
  .then(() => {
    return query.createBelongsToOne();
  })
  // Create belongs to many
  .then(() => {
    return query.createBelongsToMany();
  })
  // Attach belongs to one
  .then(() => {
    return query.attachBelongsToOne();
  })
  // Create belongs to one
  .then(() => {
    return this.ORM.query.create({
      ORM: this.ORM,
      table: query.getTable(),
      data: query.getAttributes(),
      returning: this.getReturnAttributes()
    });
  })
  // Attach
  .then((rawData) => {
    model = this.initializeModel(rawData[0], true);
  })
  // Attach belongs to many
  .then(() => {
    return query.attachBelongsToMany(model, options);
  })
  // Create has one
  .then(() => {
    return query.createHasOne(model);
  })
  // Create has many
  .then(() => {
    return query.createHasMany(model);
  })
  // Concat relationships
  .then(() => {
    model = query.concatCreatedRelationships(model);
  })
  // Post
  .then(() => {
    return this.hook(this.post.create, model);
  })
  .then(() => {
    if(initial) {
      return this.ORM.querier.commit();
    } else {
      return Promise.resolve();
    }
  })
  // Return
  .then(() => {
    return model;
  })
  // Error
  .catch((error) => {
    return this.ORM.querier.rollback().then(() => {
      console.error(error);
      return error;
    });
  });

};

ModelFactory.prototype.update = function(where, data, options) {
  // Create Query
  var queryData = new QueryObject(this.tableName, data, this.relationships, options);
  var queryWhere = new QueryObject(this.tableName, where, this.relationships, options);
  // Make search
  return this.ORM.query.update({
    ORM: this.ORM,
    table: queryWhere.getTable(),
    where: queryWhere.getAttributes(),
    data: queryData.getAttributes(),
    returning: this.getReturnAttributes()
  });
};

ModelFactory.prototype.find = function(where, options) {
  // Create Query
  var query = new QueryObject(this.tableName, where, this.relationships, options);
  // Make search
  return this.ORM.query.find({
    ORM: this.ORM,
    table: query.getTable(),
    join: query.getJoin(),
    where: query.getAttributes(),
    relationships: query.getRelationships(),
    options: query.getOptions(),
    returning: this.getReturnAttributes()
  })
  // Initialize model
  .then((rawData) => {
    return this.initializeModels(rawData);
  })
  // Include relationships
  .then((models) => {
    return (options && options.include) ?
    this.includeForMany(models, options.include) :
    models;
  });
};

ModelFactory.prototype.findOne = function(where, options) {
  options = Object.assign({}, options, { limit: 1 });
  return this.find(where, options).then((models) => {
    return models[0];
  });
};

ModelFactory.prototype.findOrCreate = function(where, options) {
  /*
  options.limit = 1;
  return this.find(where, options).then((model) => {
    if(model) {
      return model;
    } else {
      return this.create(where);
    }
  });
  */
};

ModelFactory.prototype.delete = function(where, options) {
  // Create Query
  var query = new QueryObject(this.tableName, where, this.relationships, options);
  // Make search
  return this.ORM.query.delete({
    ORM: this.ORM,
    table: query.getTable(),
    where: query.getAttributes(),
    returning: this.getReturnAttributes()
  });
};

//
// Relationship
//

ModelFactory.prototype.belongsToOne = function(modelFactoryName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    type: Relationship.Types.belongsToOne,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};

ModelFactory.prototype.belongsToMany = function(modelFactoryName, throughTableName,
  parentIdentifer_0, childIdentifer_0, parentIdentifer_1, childIdentifer_1) {
  return new Relationship({
    ORM: this.ORM,
    type: Relationship.Types.belongsToMany,
    modelFactoryName: modelFactoryName,
    parentIdentifer_0: parentIdentifer_0,
    childIdentifer_0: childIdentifer_0,
    parentIdentifer_1: parentIdentifer_1,
    childIdentifer_1: childIdentifer_1,
    throughTableName: throughTableName
  });
};

ModelFactory.prototype.hasMany = function(modelFactoryName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    type: Relationship.Types.hasMany,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};

ModelFactory.prototype.hasOne = function(modelFactoryName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    type: Relationship.Types.hasOne,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};


module.exports = ModelFactory;

==> kanso-orm-0.0.22.tgz: package/lib/model-factory.js <==
'use strict';
var QueryObject = require('./query-object');
var Relationship = require('./relationship');

function ModelFactory() {
  this.ORM = null;
  this.Model = null;
  this.tableName = '';
  this.attributes = {};
  this.relationships = {};
  this.pre = {};
  this.post = {};
  this.static = {};
  this.method = {};
}

//
// Model Operations
//

ModelFactory.prototype.extend = function(ORM, tableName, data) {
  this.ORM = ORM;
  this.tableName = tableName;
  this.Model = this.createModel(data.method);
  this.addPre(data.pre);
  this.addPost(data.post);
  this.addStatic(data.static);
  this.addTableAttributes(data.attributes);
  console.log('Creating ModelFactory: ', this.tableName, '\n');
  return this;
};

ModelFactory.prototype.createModel = function(methods) {
  //
  // Create base
  var Model = function() {};
  //
  // Add methods to model prototype
  for(var method in methods) {
    Model.prototype[method] = methods[method];
  }
  //
  // Return
  return Model;
};

ModelFactory.prototype.addPre = function(pre) {
  for(var method in pre) {
    this.pre[method] = pre[method];
  }
};

ModelFactory.prototype.addPost = function(post) {
  for(var method in post) {
    this.post[method] = post[method];
  }
};

ModelFactory.prototype.addStatic = function(statics) {
  for(var method in statics) {
    this[method] = statics[method];
  }
};

ModelFactory.prototype.addTableAttributes = function(attributes) {
  for(var attribute in attributes) {
    switch(typeof attributes[attribute]) {
      case 'object': {
        this.attributes[attribute] = attributes[attribute];
        break;
      }
      case 'function': {
        this.relationships[attribute] = attributes[attribute](this);
        break;
      }
    }
  }
};

//
// Model Instance Operations
//

ModelFactory.prototype.initializeModel = function(data, created) {
  var model = new this.Model();
  // Set all attributes
  for(var attribute in data) {
    model[attribute] = data[attribute];
  }
  if(created) {
    model.created = created;
  }
  // Set all model methods
  return model;
};

ModelFactory.prototype.initializeModels = function(data) {
  var self = this;
  var models = data.map(function(d) {
    return self.initializeModel(d);
  });
  return models;
};

//
// Attribute Operations
//

ModelFactory.prototype.getReturnAttributes = function() {
  // TEMP: Disable returning until I figure out a way to exclude instead of
  // list what to include
  /*
  var returnAttributes = [];
  for(var attribute in this.attributes) {
    if(!this.attributes[attribute].private) {
      returnAttributes.push(attribute);
    }
  }
  */
  var returnAttributes = ['*'];
  return returnAttributes;
};

//
// Relationship Operations
//

//
// Private
//

// TODO: Make private
ModelFactory.prototype.includeForMany = function(models, include) {
  return Promise.all(models.map((model) => this.includeForOne(model, include), this));
};

// TODO: Make private
ModelFactory.prototype.includeForOne = function(model, include) {
  var self = this;
  return new Promise((resolve, reject) => {
    var promises = [];
    include.forEach(function(item) {
      let relationship = self.relationships[item.relation];
      //
      // Multi level include
      let promise;
      if(item.include) {
        promise = relationship.include(item.relation, model).then((newModel) => {
          //
          // Get current tree level model
          let modelFactory = self.ORM.models[relationship.modelFactoryName];
          let models = newModel[item.relation];
          //
          // Remove current tree level and include for next set
          if(models.length) {
            return modelFactory.includeForMany(models, item.include);
          }
          else if(!models.length){
            return modelFactory.includeForOne(models, item.include);
          }
          else {
            return;
          }
        });
      }
      //
      // Single level include
      else {
        promise = relationship.include(item.relation, model);
      }
      promises.push(promise);
    });
    Promise.all(promises).then(function(values) {
      resolve(model);
    });
  });
  return Promise.resolve(model);
};

//
// Query Operations
//

ModelFactory.prototype.hook = function(hook, data) {
  return new Promise(function(resolve, reject) {
    if(hook) {
      hook(data, resolve);
    } else {
      resolve(data);
    }
  });
};

ModelFactory.prototype.create = function(where, options) {
  var model, query, initial;
  // TODO: Refactor transaction flow
  // Start transaction
  if(!options) {
    options = {};
    if(!options.transaction) {
      initial = true;
      options.transaction = this.ORM.querier.transaction();
      options.table = this.tableName;
    }
  }
  return options.transaction
  .then(() => {
    return this.hook(this.pre.create, where);
  })
  // Pre
  .then((newWhere) => {
    where = Object.assign({}, where, newWhere);
    query = new QueryObject(this.tableName, where, this.relationships, options);
  })
  // Create belongs to one
  .then(() => {
    return query.createBelongsToOne();
  })
  // Create belongs to many
  .then(() => {
    return query.createBelongsToMany();
  })
  // Attach belongs to one
  .then(() => {
    return query.attachBelongsToOne();
  })
  // Create belongs to one
  .then(() => {
    return this.ORM.query.create({
      ORM: this.ORM,
      table: query.getTable(),
      data: query.getAttributes(),
      returning: this.getReturnAttributes()
    });
  })
  // Attach
  .then((rawData) => {
    model = this.initializeModel(rawData[0], true);
  })
  // Attach belongs to many
  .then(() => {
    return query.attachBelongsToMany(model, options);
  })
  // Create has one
  .then(() => {
    return query.createHasOne(model);
  })
  // Create has many
  .then(() => {
    return query.createHasMany(model);
  })
  // Concat relationships
  .then(() => {
    model = query.concatCreatedRelationships(model);
  })
  // Post
  .then(() => {
    return this.hook(this.post.create, model);
  })
  .then(() => {
    if(initial) {
      return this.ORM.querier.commit();
    } else {
      return Promise.resolve();
    }
  })
  // Return
  .then(() => {
    return model;
  })
  // Error
  .catch((error) => {
    return this.ORM.querier.rollback().then(() => {
      console.error(error);
      return error;
    });
  });

};

ModelFactory.prototype.update = function(where, data, options) {
  return this.find(where).then((models) => {
    return Promise.all(models.map((model) => this.hook(this.pre.update, model)));
  })
  .then(() => {
    // Create Query
    var queryData = new QueryObject(this.tableName, data, this.relationships, options);
    var queryWhere = new QueryObject(this.tableName, where, this.relationships, options);
    // Make search
    return this.ORM.query.update({
      ORM: this.ORM,
      table: queryWhere.getTable(),
      where: queryWhere.getAttributes(),
      data: queryData.getAttributes(),
      returning: this.getReturnAttributes()
    });
  })
  .then((models) => {
    return Promise.all(models.map((model) => this.hook(this.post.update, model)));
  });
};

ModelFactory.prototype.find = function(where, options) {
  // Create Query
  var query = new QueryObject(this.tableName, where, this.relationships, options);
  // Make search
  return this.ORM.query.find({
    ORM: this.ORM,
    table: query.getTable(),
    join: query.getJoin(),
    where: query.getAttributes(),
    relationships: query.getRelationships(),
    options: query.getOptions(),
    returning: this.getReturnAttributes()
  })
  // Initialize model
  .then((rawData) => {
    return this.initializeModels(rawData);
  })
  // Include relationships
  .then((models) => {
    return (options && options.include) ?
    this.includeForMany(models, options.include) :
    models;
  });
};

ModelFactory.prototype.findOne = function(where, options) {
  options = Object.assign({}, options, { limit: 1 });
  return this.find(where, options).then((models) => {
    return models[0];
  });
};

ModelFactory.prototype.findOrCreate = function(where, options) {
  options = Object.assign({}, options, { limit: 1 });
  return this.findOne(where, options).then((model) => {
    if(model) {
      return model;
    } else {
      return this.create(where);
    }
  });
};

ModelFactory.prototype.delete = function(where, options) {
  return this.find(where).then((models) => {
    return Promise.all(models.map((model) => this.hook(this.pre.delete, model)));
  })
  .then(() => {
    // Create Query
    var query = new QueryObject(this.tableName, where, this.relationships, options);
    // Make search
    return this.ORM.query.delete({
      ORM: this.ORM,
      table: query.getTable(),
      where: query.getAttributes(),
      returning: this.getReturnAttributes()
    });
  })
  .then((models) => {
    return Promise.all(models.map((model) => this.hook(this.post.delete, model)));
  });
};

//
// Relationship
//

ModelFactory.prototype.belongsToOne = function(modelFactoryName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    tableName: this.tableName,
    type: Relationship.Types.belongsToOne,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};

ModelFactory.prototype.belongsToMany = function(modelFactoryName, throughTableName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    tableName: this.tableName,
    throughTableName: throughTableName,
    type: Relationship.Types.belongsToMany,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};

ModelFactory.prototype.hasMany = function(modelFactoryName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    tableName: this.tableName,
    type: Relationship.Types.hasMany,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};

ModelFactory.prototype.hasOne = function(modelFactoryName, parentIdentifer, childIdentifer) {
  return new Relationship({
    ORM: this.ORM,
    tableName: this.tableName,
    type: Relationship.Types.hasOne,
    modelFactoryName: modelFactoryName,
    parentIdentifer: parentIdentifer,
    childIdentifer: childIdentifer
  });
};

module.exports = ModelFactory;
